package kr.co.vividnext.sodalive.admin.calculate.ratio

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.member.MemberRole
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class CreatorSettlementRatioService(
    private val repository: CreatorSettlementRatioRepository,
    private val memberRepository: MemberRepository
) {
    @Transactional
    fun createCreatorSettlementRatio(request: CreateCreatorSettlementRatioRequest) {
        val creatorSettlementRatio = request.toEntity()

        val creator = memberRepository.findByIdOrNull(request.memberId)
            ?: throw SodaException("잘못된 크리에이터 입니다.")

        if (creator.role != MemberRole.CREATOR) {
            throw SodaException("잘못된 크리에이터 입니다.")
        }

        creatorSettlementRatio.member = creator
        repository.save(creatorSettlementRatio)
    }

    @Transactional(readOnly = true)
    fun getCreatorSettlementRatio(offset: Long, limit: Long): GetCreatorSettlementRatioResponse {
        val totalCount = repository.getCreatorSettlementRatioTotalCount()
        val items = repository.getCreatorSettlementRatio(offset, limit)

        return GetCreatorSettlementRatioResponse(totalCount, items)
    }
}
