package kr.co.vividnext.sodalive.admin.content.curation

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AdminContentCurationRepository :
    JpaRepository<AudioContentCuration, Long>,
    AdminContentCurationQueryRepository

interface AdminContentCurationQueryRepository {
    fun getAudioContentCurationList(): List<GetAdminContentCurationResponse>
    fun findByIdAndActive(id: Long): AudioContentCuration?
}

@Repository
class AdminContentCurationQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AdminContentCurationQueryRepository {
    override fun getAudioContentCurationList(): List<GetAdminContentCurationResponse> {
        return queryFactory
            .select(
                QGetAdminContentCurationResponse(
                    audioContentCuration.id,
                    audioContentCuration.title,
                    audioContentCuration.description,
                    audioContentCuration.isAdult
                )
            )
            .from(audioContentCuration)
            .where(audioContentCuration.isActive.isTrue)
            .orderBy(audioContentCuration.orders.asc())
            .fetch()
    }

    override fun findByIdAndActive(id: Long): AudioContentCuration? {
        return queryFactory
            .selectFrom(audioContentCuration)
            .where(
                audioContentCuration.id.eq(id)
                    .and(audioContentCuration.isActive.isTrue)
            )
            .fetchFirst()
    }
}
