package kr.co.vividnext.sodalive.admin.content.series

import com.querydsl.core.types.dsl.CaseBuilder
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.content.series.genre.QSeriesGenre.seriesGenre
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.Series
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesState
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AdminContentSeriesRepository : JpaRepository<Series, Long>, AdminContentSeriesQueryRepository

interface AdminContentSeriesQueryRepository {
    fun getSeriesTotalCount(): Int
    fun getSeriesList(
        offset: Long,
        limit: Long
    ): List<GetAdminSeriesListItem>
}

class AdminContentSeriesQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) : AdminContentSeriesQueryRepository {
    override fun getSeriesTotalCount(): Int {
        val where = series.isActive.isTrue
            .and(series.member.isNotNull)

        return queryFactory
            .select(series.id)
            .from(series)
            .where(where)
            .fetch()
            .size
    }

    override fun getSeriesList(offset: Long, limit: Long): List<GetAdminSeriesListItem> {
        val where = series.isActive.isTrue
            .and(series.member.isNotNull)

        return queryFactory
            .select(
                QGetAdminSeriesListItem(
                    series.id,
                    series.title,
                    series.introduction,
                    series.coverImage.prepend("/").prepend(coverImageHost),
                    member.nickname,
                    seriesGenre.genre,
                    seriesContent.id.count(),
                    CaseBuilder()
                        .`when`(series.state.eq(SeriesState.COMPLETE)).then("완결")
                        .`when`(series.state.eq(SeriesState.COMPLETE)).then("휴재")
                        .otherwise("연재중"),
                    series.isAdult
                )
            )
            .from(series)
            .innerJoin(series.member, member)
            .innerJoin(series.genre, seriesGenre)
            .leftJoin(seriesContent).on(series.id.eq(seriesContent.series.id))
            .leftJoin(seriesContent.content, audioContent).on(audioContent.isActive.isTrue)
            .where(where)
            .groupBy(series.id)
            .offset(offset)
            .limit(limit)
            .orderBy(series.member.id.asc(), series.orders.asc())
            .fetch()
    }
}
