package kr.co.vividnext.sodalive.admin.content.series.genre

import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminContentSeriesGenreService(private val repository: AdminContentSeriesGenreRepository) {

    @Transactional
    fun createSeriesGenre(genre: String, isAdult: Boolean) {
        val seriesGenre = SeriesGenre(genre = genre.trim(), isAdult = isAdult)
        repository.save(seriesGenre)
    }

    @Transactional
    fun modifySeriesGenre(request: ModifySeriesGenreRequest) {
        if (request.genre == null && request.isAdult == null && request.isActive == null) {
            throw SodaException("변경사항이 없습니다.")
        }

        val seriesGenre = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (request.genre != null) {
            seriesGenre.genre = request.genre
        }

        if (request.isAdult != null) {
            seriesGenre.isAdult = request.isAdult
        }

        if (request.isActive != null) {
            seriesGenre.isActive = request.isActive

            if (!request.isActive) {
                seriesGenre.genre = "${seriesGenre.genre}_deleted"
            }
        }
    }

    @Transactional
    fun modifySeriesGenreOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val seriesGenre = repository.findByIdOrNull(ids[index])

            if (seriesGenre != null) {
                seriesGenre.orders = index + 1
            }
        }
    }

    fun getSeriesGenreList(): List<GetSeriesGenreListResponse> {
        return repository.getSeriesGenreList()
    }
}
