package kr.co.vividnext.sodalive.content.donation

import kr.co.vividnext.sodalive.can.payment.CanPaymentService
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.comment.AudioContentComment
import kr.co.vividnext.sodalive.content.comment.AudioContentCommentRepository
import kr.co.vividnext.sodalive.member.Member
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AudioContentDonationService(
    private val canPaymentService: CanPaymentService,
    private val queryRepository: AudioContentRepository,
    private val commentRepository: AudioContentCommentRepository
) {
    @Transactional
    fun donation(request: AudioContentDonationRequest, member: Member) {
        if (request.donationCan < 1) throw SodaException("1캔 이상 후원하실 수 있습니다.")
        if (request.comment.isBlank()) throw SodaException("함께 보낼 메시지를 입력하세요.")

        val audioContent = queryRepository.findByIdAndActive(request.contentId)
            ?: throw SodaException("잘못된 콘텐츠 입니다.\n다시 시도해 주세요.")

        canPaymentService.spendCan(
            memberId = member.id!!,
            needCan = request.donationCan,
            canUsage = CanUsage.DONATION,
            audioContent = audioContent,
            container = request.container
        )

        val audioContentComment = AudioContentComment(
            comment = request.comment,
            donationCan = request.donationCan
        )
        audioContentComment.audioContent = audioContent
        audioContentComment.member = member
        commentRepository.save(audioContentComment)
    }
}
