package kr.co.vividnext.sodalive.notice

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.notice.QServiceNotice.serviceNotice
import org.springframework.data.domain.Pageable
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface ServiceServiceNoticeRepository : JpaRepository<ServiceNotice, Long>, ServiceNoticeQueryRepository

interface ServiceNoticeQueryRepository {
    fun getNoticeTotalCount(): Int
    fun getNoticeList(pageable: Pageable): List<ServiceNotice>
}

@Repository
class ServiceNoticeQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : ServiceNoticeQueryRepository {
    override fun getNoticeTotalCount(): Int {
        return queryFactory
            .select(serviceNotice.id)
            .from(serviceNotice)
            .where(serviceNotice.isActive.isTrue)
            .fetch()
            .size
    }

    override fun getNoticeList(pageable: Pageable): List<ServiceNotice> {
        return queryFactory
            .selectFrom(serviceNotice)
            .where(serviceNotice.isActive.isTrue)
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .orderBy(serviceNotice.id.desc())
            .fetch()
    }
}
